package org.medical.services;

import java.io.File;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import com.fasterxml.jackson.databind.ObjectMapper;

public abstract class TestHarness {
    
    private String testingEngine;
    private MedicalService[] services;
    protected double averageRequestExecutionTime; // ms
    protected double elapsedTime; // ms
    
    public TestHarness(String testingEngine) {
        this.testingEngine = testingEngine;
        ObjectMapper mapper = new ObjectMapper();
        mapper.setDateFormat(new SimpleDateFormat("yyyy-MM-dd"));
        try {
            services = mapper.readValue(new File("data/Request.json"), MedicalService[].class);
        } catch (Exception e) {
            e.printStackTrace();
        } 
    }
    
    public void test() {
        System.out.println("Warm up JVM");
        run(1000); // warm up
        run(1); 
        run(100); 
        run(1000); 
        run(2000); 
        run(10000); 
    }
    
    void run(int numberOfRepetitions) {
        int numberOfRequests = services.length * numberOfRepetitions;
        System.out.println("\nRun " + testingEngine + " against " +numberOfRequests + " requests");
        MedicalService[] batchServices = new MedicalService[numberOfRequests];
        for (int r = 0; r < numberOfRepetitions; r++) {
            for (int i = 0; i < services.length; i++) {
                batchServices[r*(services.length) + i] = services[i];
            }
        }
        long startTime = System.nanoTime();
        execute(batchServices);
        double estimatedTimeNano = System.nanoTime() - startTime;
        elapsedTime = estimatedTimeNano/1000000; // ms
        averageRequestExecutionTime = elapsedTime/numberOfRequests;
        showMetrics();
    }

    abstract public void execute(MedicalService[] services); 

    static DecimalFormat df = new DecimalFormat("0.0000");    
    public void showMetrics() {
        System.out.println("Average Request Execution Time (ms): " 
                           + df.format(averageRequestExecutionTime));
        long rps = Math.round(1000/averageRequestExecutionTime);
        System.out.println("RPS (requests per second): "+ rps);
        System.out.println("Total elapsed time (seconds): " + df.format(elapsedTime/1000));
    }
}

